
# q-expansion of newform 5618.2.a.j, downloaded from the LMFDB on 16 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5618
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [4], [0], [-4], [1], [5], [7], [-1], [5], [4], [1], [10], [-10], [-6], [0], [-6], [-4], [-4], [-15], [8], [-1], [3], [2], [17], [14], [15], [6], [-16], [-9], [5], [-10], [-8], [12], [15], [5], [-8], [2], [-19], [-2], [15], [2], [-13], [12], [-2], [0], [10], [-14], [22], [-15], [-8], [24], [-3], [4], [12], [-28], [-19], [30], [16], [-31], [-13], [10], [8], [-24], [-22], [-17], [-12], [10], [-18], [-20], [-10], [-7], [-18], [-2], [-17], [32], [-20], [-18], [24], [-21], [-12], [18], [0], [17], [2], [19], [-27], [2], [3], [-39], [-28], [3], [-12], [-33], [-11], [24], [6], [3], [-14], [-18], [-14], [-2], [-45], [0], [12], [-38], [24], [9], [-10], [-26], [8], [16], [-18], [-4], [7], [34], [16], [-42], [18], [12], [3], [-2], [16], [-2], [-43], [0], [-22], [-8], [-18], [-34], [3], [6], [16], [33], [-40], [-30], [-12], [12], [44], [12], [34], [48], [-18], [-45], [38], [-2], [-8], [-14], [18], [6], [6], [4], [4], [9], [-34], [24], [-9], [-2], [9], [-6], [32], [22], [32], [-6], [-28], [-54], [42], [23], [44], [11], [35], [28], [34], [50], [44], [-13], [-26], [-19], [-39], [37], [23], [7], [26], [23], [54], [9], [31], [0], [11], [2], [-10], [-36], [-65], [-45], [39], [-26], [-8], [59], [-24], [-44], [5], [-43], [-24], [-4], [16], [-70], [-20], [32], [26], [-47], [19], [18], [-40], [-15], [-48], [-5], [16], [42], [-20], [-24], [-22], [-5], [-2], [48], [18], [46], [28], [72], [74], [45], [6], [60], [-2], [48], [-54], [25], [74], [-63], [24], [55], [9], [-72], [50], [62], [43], [27], [-5], [33], [-18], [-18], [-59], [-45], [46], [72], [24], [66], [10], [-10], [-3], [26], [-32], [-60], [2], [38], [61], [-13], [-47], [-33], [57], [18], [-26], [75], [-5], [70], [71], [-67], [60], [31], [37], [-12], [-39], [60], [3], [-63], [83], [-6], [58], [-48], [84], [70], [-38], [-12], [-66], [39], [-46], [-42], [16], [-24], [4], [47], [-44], [-83], [2], [-80], [80], [-64], [-60], [34], [-24], [20], [24], [-66], [10], [-30], [-13], [0], [56], [68], [53], [-3], [-22], [-43], [-72], [-29], [24], [-22], [-18], [71], [-24], [-21], [-80], [50], [-2], [-14], [-34], [-33], [14], [13], [60], [56], [74], [30], [16], [88], [-9], [0], [23], [42], [-68], [-22], [-11], [52], [-29], [-31], [-9], [-28], [51], [-88], [-62], [-18], [-58], [-46], [-48], [-12], [-8], [46], [84], [49], [-66], [-62], [17], [-70], [-10], [70], [91], [13], [12], [50], [-54], [62], [30], [-86], [32], [-23], [88], [-36], [-85], [52], [-23], [-48], [6], [61], [10], [-28], [-15], [-19], [-44], [72], [28], [86], [-50], [66], [-26], [46], [46], [1], [98], [62], [-84], [-76], [29], [49], [-23], [-4], [26], [45], [-36], [32], [-76], [32], [59], [36], [-107], [-30], [-79], [19], [-25], [66], [15], [29]]
